/*
* Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
*
*/

#ifndef PIXMAPLABEL_H
#define PIXMAPLABEL_H
#include <QObject>
#include <QWidget>
#include <QLabel>
#include <QPainter>
#include <QStyle>
#include <QStyleOption>

class pixmapLabel : public QLabel
{
    Q_OBJECT
public:
    pixmapLabel();
    QList<QPixmap> m_pixmapList;
    void setPixmapList(const QList<QPixmap> &list);
protected:
    void paintEvent(QPaintEvent *event);
};

#endif // PIXMAPLABEL_H
