/*
* Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
*
*/

#ifndef CLIPBOARDDB_H
#define CLIPBOARDDB_H

#include <QObject>
#include <QtSql/QSqlDatabase>
#include <QSqlQuery>
#include <QDebug>
#include <QStandardPaths>
class clipboardDb : public QObject
{
public:
    clipboardDb(QObject *parent = nullptr);
    bool connectClipboardDb(QString DbName);
    bool creatClipboardDbTable(QString DbName);
    void insertSqlClipbarodDb(QString content, QString format, int Sequence);
    void deleteSqlClipboardDb(QString content);
    void updateSqlClipboardDb(QString content, QString format, int Sequence, QString oldcontent);
    int SelectSqlClipbaordDbId();
};

#endif // CLIPBOARDDB_H
