QPalette{background:#131314;}

/* 搜索栏的Widget */
/*QWidget#SearhWidget {
    margin-top: 0px;
    margin-right: 0px;
    margin-bottom: 0px;
    margin-left: 0px;
    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:0px;
    width:380px;
    height:40;
    background:rgba(19, 19, 20, 0);
    opacity:0;
    border: 1px solid rgba(255,255,0,1);
}*/

/* listWidget的中的widgetitem  */
QWidget#WidgetEntry {
    width:400px;
    height:42px;

    margin-top: 0px;
    margin-bottom: 0px;
    margin-right: 0px;
    margin-left: 0px;

    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:0px;
    background:rgba(255, 255, 255, 0);
    border: 0px solid rgba(255,0,0,1);
    opacity:0.08;
    border-bottom: 1px solid rgba(255,255,255,0.08);
}

/* 设置listWidget的中的widgetitem悬乎状态 */
/*QWidget#WidgetEntry:hover {
    background:rgba(255, 255, 255, 0.08);
}*/

/* 显示内容条目Widget中的置顶按钮 */
/*QWidget#WidgetEntry>QPushButton#PopButton {
    width:34px;
    height:34px;
    margin-bottom: 0px;
    margin-top: 0px;
    margin-right: 0px;
    margin-left: 0px;
    border: 0px solid rgba(255,0,0,1);
    background:rgba(255,255,255,30);
    opacity:0.12;
    border-radius:3px;
}


QWidget#WidgetEntry>QPushButton#PopButton:hover {
    background-color:rgba(255,255,255,36);
    opacity:0.2;
    border: 0px solid rgba(255,0,0,1);
}


QWidget#WidgetEntry>QPushButton#PopButton:pressed {
    background-color:rgba(255,255,255,0.08);
    opacity:0.08;
    border: 0px solid rgba(255,0,0,1);
}*/

/* 显示内容条目Widget中的编辑按钮 */
/*QWidget#WidgetEntry>QPushButton#EditButon {
    width:34px;
    height:34px;
    margin-bottom: 0px;
    margin-top: 0px;
    margin-right: 0px;
    margin-left: 0px;
    border: 0px solid rgba(255,0,0,1);
    background:rgba(255,255,255,30);
    opacity:0.12;
    border-radius:3px;
}*/

/* 显示内容条目Widget中的置编辑钮悬乎 */
/*QWidget#WidgetEntry>QPushButton#EditButon:hover {
    background-color:rgba(255,255,255,36);
    border-radius:3px;
    opacity:0.2;
    border: 0px solid rgba(255,0,0,1);
}*/

/* 显示内容条目Widget中的编辑按钮按下 */
/*QWidget#WidgetEntry>QPushButton#EditButon:pressed {
    background-color:rgba(255,255,255,25);
    border-radius:3px;
    opacity:0.08;
    border: 0px solid rgba(255,0,0,1);
}*/

/* 显示内容条目Widget中的删除按钮 */
/*QWidget#WidgetEntry>QPushButton#RemoveButton {
    width: 34px;
    height: 34px;
    margin-bottom: 0px;
    margin-top: 0px;
    margin-right: 0px;
    margin-left: 0px;
    border: 0px solid rgba(255,0,0,1);
    background:rgba(255,255,255,30);
    opacity: 0.12;
    border-radius: 3px;
}*/

/* 显示内容条目Widget中的删除按钮悬乎 */
/*QWidget#WidgetEntry>QPushButton#RemoveButton:hover {
    background-color:rgba(255,255,255,36);
    opacity:0.2;
    border-radius:3px;
    border: 0px solid rgba(255,0,0,1);
}*/

/* 显示内容条目Widget中的删除按钮按下 */
/*QWidget#WidgetEntry>QPushButton#RemoveButton:pressed {
    background-color:rgba(255,255,255,0.08);
    opacity:0.08;
    border: 0px solid rgba(255,0,0,1);
}*/

/* 清空所有的WidgetItem按扭 */
/*QWidget#SearhWidget>QPushButton#CleanList {
    width:80px;
    height:34px;
    margin-top:0px;
    margin-left:0px;
    margin-bottom:0px;
    margin-right:0px;

    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:0px;
    border-radius:4px;
    font-size:14px;
    font-family:Noto Sans CJK SC;
    font-weight:400;
    color:rgba(255,255,255,1);
    line-height:24px;
    opacity:0.91;
    border:0px solid rgba(255,0,0,1);
}*/

/* 清空所有的WidgetItem按扭悬浮时的效果 */
/*QWidget#SearhWidget>QPushButton#CleanList:hover{
    background-color:rgba(255,255,255,36);
    border-style:inset;

    border: 0px solid rgba(255,0,0,1);
}*/

/* 清空所有的WidgetItem按扭按下时的效果 */
/*QWidget#SearhWidget>QPushButton#CleanList:pressed{
    background-color:rgba(255,255,255,25);
    border-style:inset;

    border: 0px solid rgba(255,0,0,1);
}*/

/* 清空所有的QlineEdit按扭 */
QLineEdit#SearchLabel>QPushButton#ClearTextButton {
    margin-top:0px;
    margin-left:0px;
    margin-bottom:2px;
    margin-right:0px;

    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:0px;
    border-radius:0.5px;
    border:0px solid rgba(255,255,0,1);
}

/* 查找栏的Lable */
/*QWidget#SearhWidget>QLineEdit#SearchLabel {
    width:280px;
    height:30px;
    margin-top: 0px;
    margin-right: 0px;
    margin-bottom: 0px;
    margin-left: 0px;
    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:3px;
    background:rgba(0,0,0,0.2);
    border:0px solid rgba(61,107,229,1);
    border-radius:4px;

    font-size:14px;
    font-family:Noto Sans CJK SC;
    font-weight:400;
    color:rgba(255,255,255,1);
    line-height:24px;
    opacity:0.91;

}*/

/* 查找栏的Lable悬乎状态 */
/*QWidget#SearhWidget>QLineEdit#SearchLabel:hover {
    border-style:inset;
    border: 1px solid rgba(61,107,229,1);
    border-radius:4px;
}*/

/* 查找栏的Lable焦点状态 */
/*QWidget#SearhWidget>QLineEdit#SearchLabel:focus {
    border-style:inset;
    border: 1px solid rgba(28, 47, 146, 1);
    border-radius:4px;
}*/

/* 显示所有剪贴内容的WidgetList大小 */
QListWidget#ShortcutOperationList {
    width:400px;
    height:760px;

    margin-right: 0px;
    margin-bottom: 0px;
    margin-left: 0px;
    margin-top: 0px;

    padding-top:0px;
    padding-bottom:0px;
    padding-left:0px;
    padding-right:0px;
    background:rgba(19, 19, 20, 0);
    opacity:0;
    border:0px;
}

QListWidget#SearchWidgetListWidget {
    width:400px;
    height:34px;

    margin-right: 0px;
    margin-bottom: 0px;
    margin-left: 0px;
    margin-top: 2px;

    padding-top:0px;
    padding-bottom:0px;
    padding-left:0px;
    padding-right:0px;
    background:rgba(19, 19, 20, 0);
    opacity:0;
    border: 0px solid rgba(255,0,255,0);
    border:0px;
}

/* widget条目的Lable */
/*QWidget#WidgetEntry>QLabel#EntryLable {
    width:200px;
    height:34px;
    font-size:15px;
    font-weight:400;

    margin-top: 0px;
    margin-right: 0px;
    margin-bottom: 0px;
    margin-left: 0px;

    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:3px;
    font-size:14px;
    font-family:Noto Sans CJK SC;
    font-weight:400;
    color:rgba(255,255,255,1);
    line-height:40px;
    opacity:0.91;
}*/

QListWidget:Item{
    margin-top: 0px;
    margin-right: 0px;
    margin-bottom: 0px;
    margin-left: 0px;

    padding-right: 0px;
    padding-top: 0px;
    padding-bottom:0px;
    padding-left:0px;
    background:rgba(19, 19, 20, 0);
    border: 0px solid rgba(255,255,0,1);
}

/* 无剪贴板内容字样 */
/*QLabel#SideBarClipboardLable{
    width:70px;
    height:14px;
    margin-top:50px;
    margin-left:165px;
    margin-bottom:100px;
    margin-right:132px;
    font-size:14px;
    font-family:Noto Sans CJK SC;
    font-weight:400;
    color:rgba(255,255,255,1);
    line-height:28px;
    opacity:0.57;
}*/
